
# q-expansion of newform 3762.2.a.bi, downloaded from the LMFDB on 17 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 3762
weight = 2
poly_data = [-12, 35, 14, -12, -2, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0], 1], [[0, 1, 0, 0, 0], 1], [[-5, -1, 1, 0, 0], 1], [[6, -7, -2, 1, 0], 2], [[12, 10, -9, -2, 1], 2]]

hecke_ring_character_values = None
aps_data = [[-1, 0, 0, 0, 0], [0, 0, 0, 0, 0], [-1, 1, 0, 0, 0], [1, 0, 0, -1, 0], [-1, 0, 0, 0, 0], [1, 0, 1, 0, 0], [0, -1, 0, -1, 1], [1, 0, 0, 0, 0], [-1, 0, 1, 0, 0], [-2, 1, 1, 0, 0], [2, 0, 0, 0, 1], [2, 1, -2, -1, 0], [0, 0, -1, -1, -1], [2, 1, -1, 0, 0], [-1, 0, 1, 0, 0], [1, 0, -1, 0, -1], [-2, 0, 1, -1, 1], [3, 1, 0, 2, 1], [0, 3, 0, 1, -1], [3, 0, 0, -3, 0], [2, 2, 1, 3, 1], [3, 2, 1, -2, 0], [0, 0, -1, -1, -1], [0, -3, 1, -2, 2], [4, -2, 1, 1, -1], [2, 0, -3, -1, 0], [2, 0, -2, -2, 1], [4, -1, 0, 3, 1], [3, 0, -1, 0, -2], [0, -3, -2, -5, 1], [5, -1, 0, 2, 0], [4, -4, 1, 1, 1], [2, 4, 0, 0, -2], [-4, 5, 2, 1, -1], [0, -2, 1, -1, 0], [-1, 5, -2, 0, 0], [6, -4, 0, 0, 0], [4, -2, 1, 1, 1], [6, -2, 0, 4, 0], [0, 1, -1, 0, 2], [-2, 4, 0, 2, 0], [2, -1, 2, 1, 0], [-5, -2, -1, 4, -2], [8, -3, -1, -4, 0], [0, 0, 1, 1, -2], [-4, -3, -1, 2, 0], [8, 0, 2, -4, 0], [8, -4, 0, 2, 1], [4, 3, 1, 2, 2], [4, -4, -2, -4, 2], [6, -4, 0, -4, -2], [6, -5, -1, 0, 1], [0, -5, 1, -6, 0], [14, -1, 1, -4, 2], [-12, 3, 3, 6, -2], [6, 4, 0, 4, -1], [5, 6, 1, 0, -1], [5, -4, -2, -3, 0], [3, 3, -2, 2, 1], [4, 0, 0, -2, 4], [-6, 6, -1, 3, 1], [0, 1, -3, -2, -2], [2, -2, 2, 0, 0], [-3, 6, 1, 4, 2], [-4, -7, -2, -3, 3], [7, 0, 1, 8, 1], [-12, 3, -1, 0, 0], [2, 7, -1, 6, 0], [4, -4, -1, -1, -3], [-7, 1, 8, 0, 3], [-4, 4, -4, 2, 0], [12, 0, -6, 0, -1], [-8, 3, 3, -4, 0], [1, 2, 1, 2, 2], [-4, 1, 3, -2, 2], [15, -3, 1, 1, 4], [-7, -2, 3, 0, 2], [-2, -8, 2, 2, -2], [-2, 3, -2, -1, -1], [8, 2, 2, 4, 0], [-4, -9, 1, 0, -2], [10, -6, -1, -5, 2], [10, -2, 0, -4, -2], [6, 2, 0, 6, 0], [1, -7, -2, 2, 0], [-4, 3, 3, -10, 0], [10, 0, -2, 2, -2], [4, -2, 4, 4, 2], [6, -2, 1, 5, 4], [0, -7, 1, -8, 2], [14, -2, 4, -2, -2], [4, 5, -5, 4, -3], [20, -6, 2, 2, 3], [4, 0, -4, 0, -2], [-4, 2, 2, 4, -2], [18, 8, -4, 4, -1], [1, -6, 5, 0, -1], [-10, 9, -1, -2, 0], [10, -2, -2, -2, 2], [1, -6, 1, 0, 3], [-2, 0, -8, 0, 0], [-4, 0, 7, 3, 0], [8, 1, 6, 3, 1], [-10, 10, 3, 3, -3], [-4, 5, 3, 2, -4], [-2, 9, -6, -1, -1], [0, -5, 1, -4, 2], [16, -6, -3, -11, 3], [-9, -3, -5, -5, 4], [-6, 6, 0, 4, 0], [-3, -6, 1, -4, 6], [-5, -6, 3, -4, 5], [-4, 0, -2, 0, -2], [-12, 2, 1, 1, -1], [10, -3, 1, 6, 0], [2, -8, 4, 4, 0], [0, 4, -4, 4, -4], [15, -8, -3, -8, 4], [-33, 2, -3, -4, -2], [0, 0, 0, 0, -2], [-8, -2, 3, -3, 2], [4, -5, 1, -8, 0], [2, -8, -2, -2, 0], [-2, -6, 0, 4, 2], [-12, 0, -5, 5, -5], [-4, -10, 1, -1, -2], [-34, -6, 2, -4, 2], [1, 0, -9, -8, 2], [-24, 1, -1, -2, -2], [-7, -1, 4, -6, 3], [-6, 10, 1, -3, -1], [-4, -4, 8, 0, 0], [8, -4, 0, 2, -1], [-2, 6, 6, 8, -2], [0, -5, 0, 7, 1], [-28, 2, 3, -1, -1], [-11, 4, -3, 2, -5], [-4, 2, 0, -4, 4], [-11, 6, -1, 6, 1], [-18, -8, 2, 6, -2], [-6, 4, -4, -2, -2], [8, 4, -3, 9, 0], [-16, -1, -3, 2, 0], [4, 0, 1, -1, 1], [6, -3, 2, -9, -2], [5, 7, 5, 5, -2], [-9, 13, 0, 2, -1], [2, -6, -2, -12, 0], [-12, -4, -2, -2, 4], [-3, 4, 8, 3, -2], [-15, 4, 3, 2, -2], [-20, 6, -8, -4, -2], [0, -6, -1, 3, -3], [6, 2, -4, 4, 0], [-12, -7, 4, -5, 1], [11, -1, 1, -1, -2], [9, -6, -6, -5, 0], [-18, 0, 0, 0, -2], [-10, -2, -1, -3, -1], [-28, -7, 1, 2, 0], [-6, -3, 5, -4, 2], [16, 0, 1, -7, 5], [-5, -2, -8, 7, -4], [-2, 4, 4, 6, 0], [-6, 6, -3, 3, 1], [3, 8, -8, -3, -2], [-20, -4, 0, -2, -3], [-23, -2, -3, 0, -3], [8, 3, -5, 4, -6], [-33, 9, 2, 2, 2], [8, -4, -9, 1, -5], [-6, 1, -2, 3, 6], [2, -7, -5, -10, 3], [-6, -2, 7, -3, -1], [-6, -14, 2, -8, 3], [-20, 6, 4, -4, 2], [-24, -7, 0, 3, -3], [-5, -3, -4, 0, 4], [19, -6, -3, 2, 0], [-23, 2, -5, 2, -9], [-10, 9, -3, 6, -6], [16, 5, 5, -4, 2], [16, -6, -3, -1, 2], [12, 2, 9, -1, -1], [-13, 2, -1, 0, 2], [-7, -3, -2, 0, 4], [13, -6, 3, -4, -4], [8, -9, -2, -5, 3], [0, -9, -3, 4, 4], [22, 3, 5, -6, -3], [-8, 0, 3, 5, 1], [4, 2, -1, 5, 1], [8, 3, 1, -2, 6], [-15, -4, 3, -4, -1], [28, -2, 2, -8, 4], [-28, 3, -2, -3, -1], [18, 0, -9, -5, 1], [10, 4, 8, -4, 0], [14, -11, -3, 0, -4], [8, 12, -2, 0, -1], [-24, 5, -3, 2, -4], [-9, 1, 2, 4, -2], [14, -7, 6, 11, 0], [2, 9, 5, 8, -4], [4, 1, 5, 10, 4], [-13, 9, 4, 0, -2], [-4, -4, -10, -2, 0], [-4, 3, -8, -9, 3], [20, -7, 2, 3, 3], [20, -9, -6, -9, -1], [-16, 18, 0, 0, -6], [-11, 10, 5, 10, -2], [8, -8, 6, -2, 0], [16, 4, -3, 5, 1], [14, 2, 2, 6, -2], [0, -18, 1, -7, 3], [19, -4, -2, -13, 2], [8, 4, -6, 0, 2], [7, 1, -1, 7, 0], [-5, -1, 12, 6, 0], [-3, 6, -3, -2, 2], [19, -17, 4, 4, 2], [-2, 5, 5, -4, 2], [26, 0, 0, -6, -2], [0, 4, -3, 7, -1], [-11, 11, -8, -10, 1], [-12, -8, -6, 4, 2], [29, 1, -3, 3, 2], [-13, -10, 1, 6, 0], [0, -14, 0, -2, 2], [-6, 4, 2, -2, 0], [28, -8, -2, 4, 0], [1, 7, -2, 4, -10], [-2, 0, -1, 9, -5], [24, 8, 1, -5, 7], [32, 11, -3, 4, -3], [0, -1, 8, 5, -5], [-4, 8, -11, -7, 4], [-10, 1, -3, 6, -4], [1, -9, 7, -1, -2], [8, -8, -11, 1, -3], [2, 8, 4, -6, 0], [30, 4, 0, 2, -2], [18, 5, -10, -1, -2], [18, -12, -7, -13, 1], [-10, 0, 6, 2, 1], [0, 3, 3, 4, -8], [12, -6, 3, -3, 5], [20, 8, 0, 8, 0], [8, -1, 1, -4, -7], [39, -3, -4, 6, -1], [-26, 3, -7, 6, -2], [25, 0, -1, 0, 0], [-24, 6, -7, -3, 1], [10, 10, 1, -3, 0], [8, -4, 1, 17, -1], [-14, -4, 4, -4, 4], [-6, 16, 2, 16, -8], [27, -6, 1, -2, -5], [28, -2, 5, -7, 1], [37, -15, -8, -12, 4], [-8, -1, -1, -4, 6], [-15, 6, -1, 6, 5], [25, -2, -5, 4, 2], [6, -4, 0, -10, 6], [-28, 6, 9, 3, 1], [-13, -20, 3, -6, 5], [28, -6, 8, 6, 6], [16, -10, -2, -4, 0], [-6, -10, -5, -9, 8], [28, -10, 1, -1, -2], [-4, 2, -3, 11, -3], [8, 1, -1, 12, 0], [7, -2, -4, 11, -4], [-2, 2, 10, -10, 6], [0, -12, -6, -2, -4], [-22, 18, -2, 0, -4], [-27, -8, -1, 10, -2], [10, -19, -5, -10, 6], [-16, 12, 2, 10, 2], [-22, -10, 4, -4, 4], [14, -9, -3, 0, -6], [44, -7, 5, 6, 1], [-35, 4, -3, 0, -2], [-4, -15, 0, -9, 9], [35, -8, 1, 4, 0], [2, 13, 6, 7, 3], [14, -16, 2, 6, 2], [-2, 0, 6, -16, 4], [-8, 0, -16, 6, -4], [-21, 3, -4, 2, -8], [-38, 7, 5, 4, -2], [12, 12, -4, 2, 0], [-20, 1, -11, 0, 0], [35, -8, -7, -6, 0], [6, 6, -7, 5, -10], [-24, -10, -2, 10, -5], [5, -2, -5, 16, -4], [-16, -12, 2, 10, -6], [2, 2, -11, -3, -5], [-28, 14, 8, 16, -6], [-18, 1, -9, -2, -4], [5, -8, 4, 11, -4], [-8, -4, -9, 3, -5], [28, 8, 1, 11, 1], [-2, 0, 8, 4, -2], [-16, -8, -8, 4, 8], [22, -3, 14, 7, 2], [-33, 4, -3, 4, 0], [33, -13, 4, 10, 1], [4, 1, 9, -10, -1], [-37, 11, -4, -6, -2], [-2, -4, 8, -4, -4], [44, 11, -7, 10, -4], [26, -1, 7, 8, 1], [-10, -13, -3, 2, 6], [26, -14, -8, -2, -2], [44, -6, 4, -6, 10], [-10, 7, 3, 4, -2], [-34, 15, 5, -2, -2], [-26, 4, 12, 0, 4], [6, 1, 4, -15, 5], [4, -9, -5, -22, 4], [36, -2, 4, 12, 3], [-24, -14, -1, 3, 3], [12, 16, 1, -9, 5], [-32, -18, 3, -7, 5], [-16, 13, 7, 2, -4], [36, 14, -2, 0, 2], [4, -12, -7, 9, 2], [3, -15, 2, 0, -3], [9, -4, -3, -10, -1], [-19, 12, -6, 9, 0], [-28, 4, -6, 6, 2], [14, 5, -2, 3, 3], [-10, -5, 1, -14, 4], [2, -1, -2, 21, -2], [-36, 18, 0, 0, 2], [32, -3, -6, -3, -5], [15, -10, 6, -3, 6], [-32, 6, 7, 9, 2], [2, 14, 4, 8, 8], [-4, -12, -6, 8, -4], [-6, 20, 4, -8, 1], [-35, 14, -1, 2, -5], [24, -9, 4, -5, 7], [2, 16, 8, 0, 0], [28, -12, -6, -10, -4], [12, 0, 14, -10, 2], [1, -2, 5, 10, -8], [10, 0, -6, 8, 0], [54, -14, -2, 0, -2], [-27, -7, 0, -10, -4], [11, 17, -8, -6, 0], [3, -12, -7, -6, -8], [-22, 16, 8, -4, 4], [33, -6, -9, -6, -4], [-12, 4, 2, -6, 4], [18, -19, -5, -6, -4], [-11, -1, -11, 1, -2], [3, 7, -10, -2, -5], [28, -1, 2, 17, 5], [-1, 17, 1, 17, 2], [-32, -3, -7, -12, 0], [-16, 12, -3, -9, -3], [18, -12, 0, -8, -8], [48, -6, -7, -13, 2], [27, 10, -5, 12, 0], [-16, 11, -1, 4, -6], [2, 7, -15, 0, 2], [8, 20, -4, -8, 0], [-4, 1, 1, 10, 7], [-2, 4, 12, 8, 2], [-21, -2, 4, 15, -10], [28, 4, -8, -2, -2], [-20, -10, 0, -6, 10], [-14, -3, 1, -4, -5], [-22, -6, -9, -15, 3], [-10, -4, -4, 12, 0], [0, 17, 11, 20, -8], [25, -10, 13, 4, -1], [-10, -12, 8, -2, -2], [15, 0, 8, -3, 4], [0, -9, 10, 1, 1], [-8, -6, -8, -12, -4], [9, 11, -9, -1, -4], [46, -17, -3, 0, 2], [18, 0, -18, -2, -2], [26, -19, 7, 0, 0], [13, -4, 0, 15, -4], [16, -6, -7, 13, 1], [-1, 14, -1, 12, 3], [-34, -7, -5, -4, -6], [-44, 6, -7, 1, 5], [29, -4, 14, -3, 0], [16, 4, 2, -4, 2], [-33, 30, 7, 8, -2], [14, 6, -5, 21, -1], [36, -9, 7, 8, 0], [-8, 12, 11, 15, 2], [-55, 2, -3, 2, 6], [-14, 0, -2, 2, 8], [7, -5, -4, -16, 8], [30, -15, -9, -6, 0], [-40, 13, 3, -12, -4], [-23, -8, 4, -17, 2], [8, -14, 10, -10, 6], [42, -2, -6, 4, -4], [12, 9, 12, 7, -5], [-38, 1, 7, -6, 0], [30, -4, -8, 4, -2], [-10, 14, -5, -1, -4], [-24, 6, 1, -5, -1], [4, 2, 9, 7, -5], [-2, 6, -4, -2, -10], [-39, 18, 11, 8, -4], [2, -4, -8, 14, 2], [-34, 2, -6, -4, -9], [18, 2, -16, 16, -4], [20, -24, -4, -8, -4], [7, -12, -1, 6, 11], [-17, 14, 13, 8, 6], [25, -4, 3, 12, 6], [20, 4, 10, 10, 0], [-4, 0, -9, 9, -5], [-11, 10, -11, 0, -1], [-36, -14, 3, -11, 5], [-26, 7, -7, -2, 2], [0, 0, -9, 25, -3], [14, -1, 13, 14, -3]]
